unsigned const char Gen_[] = {0xD7,0x11,0xCD,0x5D,0x1B,0x5E,0xDE,0x15,0xDF,0x5F};
        // define digits (bits-segments: 0-b; 1-f; 2-a; 3-g; 4-c; 5-h; 6-d; 7-e)

unsigned char Out_symbol[8];         // signs to OUT
unsigned char Sign[4] = {0,0,0,0};
volatile unsigned char Counter = 0;  // Counts 16ms intervals (clear each 4s)

#define _u  0xD0                     //----- define symbols -------------------
#define _Bottom 0x40
#define _E  0xCE
#define _r  0x88
#define _Top  0x04
#define _Null 0xD7



#define out_port GPIO
#define out_tris TRISIO
#define set(x,y)  x = x | (1<<y)         // set bit
#define clear(x,y)  x = x &~(1<<y)       // clear bit
#define int_off  INTCON.GIE = 0
#define int_on  INTCON.GIE = 1

#define INTERVAL_BIT Counter.F1          // mesure interval - 0.25s
//#define INTERVAL_BIT Counter.F2        // mesure interval - 0.5s
//#define INTERVAL_BIT Counter.F3        // mesure interval - 1s






//************* PROCEDURE OF OPERATION ADC IN SLEEP MODE *********************
// By Reg. ANCEL or ADCON1 it is must be selected analog or digital ports AN0-AN7
// Input parameters: AN_x - AN-input number (0-7); Vr - VREF or Udd (1 or 0)

unsigned int ADC_sleep(unsigned char AN_x, unsigned char Vr) {
 unsigned char *ANCEL_ADCON1_addr = 0x9F;
 unsigned char x, Int_temp;

 PIE1.ADIE = 1;  // enable ADC int.

 (*ANCEL_ADCON1_addr) |= 0b01110000; // internal osc.

 x = (AN_x << 2)+1;    //
 x.F7 = 1;             //
 if (Vr==1) x.F6 = 1;  //

 Int_temp = INTCON;
 INTCON.GIE = 0; // Global int. off

 ADCON0 = x;           //   set ANx and Vpef and ADC-on

 asm {
    movlw 0x0F
    addlw 0xFF
    btfss STATUS, Z
    goto  $-2      //pause 62 cicles

    bsf   INTCON,PEIE   //
    bcf   PIR1,ADIF     // enable ADC int.
    bsf          ADCON0,GO     // ADC start
    sleep
    bcf        INTCON,PEIE
    }

 INTCON = Int_temp;
 return (unsigned int)(ADRESH <<8)+ADRESL;
 }




// ********** SYNCRONIZATION WITH INTERRUPS ************************************
// *****************************************************************************
void Int_sync(void){
  int_off;
  while (INTCON.T0IF==0);
  int_on;
 }

//*****************************************************************************




//********************** PROC. PREPARE DATA TO IND. ***************************
// Write 1-st symbol to 0 bits of Out_symbol 2-nd to 1 bits ...
//*****************************************************************************

void Data_to_ind () {
unsigned char i, j, s1, s2;

    Int_sync();                          // Sync. with interrupts
    for (j=0; j<4; j++) {                // 4 symbols
      s1=Sign[j];
      for (i=0; i<8; i++) {              // for each bit of symbol
        s2 = Out_symbol[i];
        if (j==0) s2=0;                  // at first - clear Out_symbol
        s2 <<=1;                         // next symbol -next bit (7-th bit-first)
        if (s1.F7==1) s2.F0 = 1;         // set bit if same bit of symbol is 1
        Out_symbol[i]=s2;
        s1 <<= 1;                        // next bit of symbol
      }
    }

}

//******************************************************************************




//********** BIN to BCD (0 - 999)  PROCEDURE ***********************************
// In parameter - int, index to array Sign[] (symbols to segments)
// writes digits to out
//******************************************************************************
void Int_to_ind(unsigned int In_int) {
  unsigned char s;

  s = (unsigned char)(In_int / 100);  // handreds
  if (s>9) return ;

  Sign[0] = Gen_[s];
  s = (unsigned char)(In_int % 100);
  Sign[1] = Gen_[(unsigned char)(s / 10)];   // decades
  Sign[2] = Gen_[(unsigned char)(s % 10)];  // units
 }
 //*****************************************************************************







// *****************************************************************************
// ********************* DIVISION UNSIGNED LONG BY UNSIGNED LONG ***************
// In: unsigned long dividend , unsigned long divisor, char ('/' or '%')
// Out: unsigned long
// CALLING: for devision - x = Dev_U_Long((long)a, (long)b, '/')
//          for remainding of devision -  x = Dev_U_Long((long)a, (long)b, '%')
// warning!!  Divisor must be tested for zero!
// *****************************************************************************

unsigned long Dev_U_Long (unsigned long Dividend, unsigned long Divisor, char x) {

  asm {
        clrf    R0
        clrf    R0+1
        clrf    R0+2
        clrf    R0+3

        clrf    R5

        movlw   32
        movwf   R4

DIVIDE_LOOP_32by32:
        rlf     FARG_Dev_U_Long_Dividend+0,F
        rlf     FARG_Dev_U_Long_Dividend+1,F
        rlf     FARG_Dev_U_Long_Dividend+2,F
        rlf     FARG_Dev_U_Long_Dividend+3,F
        rlf     R0+0,F
        rlf     R0+1,F
        rlf     R0+2,F
        rlf     R0+3,F

        rlf     R5, f

        movf    FARG_Dev_U_Long_Divisor+0,W
        btfsc   R5, 7
        goto    Div32by32_add


        subwf   R0+0,F

        movf    FARG_Dev_U_Long_Divisor+1,W
        btfss   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+1,W
        subwf   R0+1,F

        movf    FARG_Dev_U_Long_Divisor+2,W
        btfss   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+2,W
        subwf   R0+2,F

        movf    FARG_Dev_U_Long_Divisor+3,W
        btfss   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+3,W
        subwf   R0+3,F

        movlw   1
        btfss   STATUS,C
        subwf   R5, f
        goto    DIVIDE_SKIP_32by32
Div32by32_add:

        addwf   R0+0,F

        movf    FARG_Dev_U_Long_Divisor+1,W
        btfsc   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+1,W
        addwf   R0+1,F

        movf    FARG_Dev_U_Long_Divisor+2,W
        btfsc   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+2,W
        addwf   R0+2,F

        movf    FARG_Dev_U_Long_Divisor+3,W
        btfsc   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+3,W
        addwf   R0+3,F

        movlw   1
        btfsc   STATUS,C
        addwf   R5, f

DIVIDE_SKIP_32by32:
        decfsz  R4,F
        goto    DIVIDE_LOOP_32by32

        rlf     FARG_Dev_U_Long_Dividend+0,F
        rlf     FARG_Dev_U_Long_Dividend+1,F
        rlf     FARG_Dev_U_Long_Dividend+2,F
        rlf     FARG_Dev_U_Long_Dividend+3,F

        btfsc   FARG_Dev_U_Long_Dividend+0,0
        goto    DS_0

        movf    FARG_Dev_U_Long_Divisor+0,W
        addwf   R0+0,F

        movf    FARG_Dev_U_Long_Divisor+1,W
        btfsc   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+1,W
        addwf   R0+1,F

        movf    FARG_Dev_U_Long_Divisor+2,W
        btfsc   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+2,W
        addwf   R0+2,F

        movf    FARG_Dev_U_Long_Divisor+3,W
        btfsc   STATUS,C
        incfsz  FARG_Dev_U_Long_Divisor+3,W
        addwf   R0+3,F

DS_0:
        movf    FARG_Dev_U_Long_x,0
        xorlw   0x25
        btfss   STATUS,2
        goto    DS_1

        movf    R0+3,0
        movwf   FARG_Dev_U_Long_Dividend+3
        movf    R0+2,0
        movwf   FARG_Dev_U_Long_Dividend+2
        movf    R0+1,0
        movwf   FARG_Dev_U_Long_Dividend+1
        movf    R0,0
        movwf   FARG_Dev_U_Long_Dividend

DS_1:
        nop

  }
  return Dividend;
}

// *****************************************************************************










// ******************** THE MAIN PROGRAMM **************************************



void main() {

  char Num, Num1, i;
  unsigned int x, Corr_, Calib_x;

  Corr_ = Eeprom_Read(1)<<8;              //
  Corr_ = Corr_ + Eeprom_Read(0);         // Read correction number

  Num = Eeprom_Read(3);
  if (Num > 4) Num = 4;

  int_off;
  Eeprom_Write(3, Num);
  int_on;

  out_tris = 0b11001001 ;
  out_port = 0b00110110 ;  // Ind. --> off
  OPTION_REG = 0b10000010; // tmr0 prescaler 8, pull_ap off
  ANSEL = 1; // AN0 - analog

  INTCON = 0b10100000;     // int_TMR0 on


                           // ----------   CALIBRATION  -----------------------
  if (out_port.F3==1) {    // if calibration pins is not in junction (ON)
     if (Num==0) Num = 4;
     Num1 = Num;
     while (1) {
       if (Counter.F6==0)  Sign[3] = _u;//
         else  Sign[3] = 0;             // Symbol "u" is flash

       Calib_x = (unsigned int)Num*200;
       Int_to_ind(Calib_x);             // BIN to BCD ( to out in 7-seg. code)
       set(Sign[1],5);                  // comma at 2-nd digit
       Data_to_ind();                   // Prepare data to leadout in interrupt

       if (out_port.F3==0) break;      // exit if pins are joining
       if (Num1==Num) {
             if (--Num1 == 0) Num1==4; // 4,3,2,1,4,3....
             int_off;
             Eeprom_Write(3, Num1);    // save next
             int_on;
          }
       }

     x = ADC_sleep(0,0);              //    AD conversion (AN0, Vref = Upp)

     if ((x>(Calib_x+200))||(x<(Calib_x-150))) {
         Sign[0] = _E;                 //
         Sign[1] = _r;                 //
         Sign[2] = _r;                 //
         Sign[3] = _Bottom;            // if 256<x<1023 --> Error

         Data_to_ind();                // Prepare data to leadout in interrupt

          while (1);
        }

     Corr_= Dev_U_Long((unsigned long)Calib_x*2048, x, '/'); // Corr_= Calib_x * 2048 / x

     int_off;
     Eeprom_Write(3, 0);               // calibration is done

     Eeprom_Write(0, (char)Corr_);    //
     Eeprom_Write(1, Corr_>>8);       // Save corr. int
     int_on;

     }

  while (INTERVAL_BIT==0);           //
  while (INTERVAL_BIT==1);           // pause 50-100 ms

  Sign[3] = _u;
  if (Eeprom_Read(3)!= 0) {               // if don't calibrated
      set(Sign[3],5);                     // comma at 4-th digit
      Corr_ = 2048;                       // Corr_ is average
      }


  while (1){
   x = 0;
   for (i=0; i<8; i++){

    while (INTERVAL_BIT==0);           // 1 measure for (0.032-0.128)s
        x += ADC_sleep(0,0);           //  SUM of 8  AD conversions
    while (INTERVAL_BIT==1);           // wait the next  interval
    }

   x = ((unsigned long)(Corr_)*x+8192)>>8; // Average x with correction
   x >>= 6;                                //    and rounding

   Int_to_ind(x);                      // BIN to BCD ( 7-seg. code)
   set(Sign[1],5);                     // comma at 2-nd digit
   if (Sign[0]==_Null) Sign[0] = 0;    // 1-st null blanking

   if (x>800) Sign[2] = _Top;          // Overvoltage - top drow

   Data_to_ind();                      // Prepare data to leadout in interrupt


  }

}


// *********************** END OF MAIN *****************************************






//**************************** INTERRUPTS **************************************
void interrupt(void) {
  if (INTCON.TMR0IF == 1){
      unsigned char To_port=0, To_ind, i;
      static unsigned char Col_ = 0;      // column of indicator

      Col_++;                             //
      if (Col_.F3 != 0) {                 //
         Col_ = 0;                        // 0,1,2...7,0,1,2,...
         Counter += 1;                  // counter increment at 2ms*8 = 16ms
         }

      //out_port = 0;                       //  REG-->0
      //out_port =  0b00110000;             // ind. off , (REGout = 00000000)
      asm {
         movlw 0b00110000  // same --> out_port = 0; out_port = 0b00110000
         clrf out_port     // but port is in null state within 1 cycle instead 2
         movwf out_port    // it decreases lighting of inactive segments
         }

                         // Set segment to REG
      for (i=0; i<8; i++) {
         To_port = 0b00110000;
         if (i==Col_) To_port.F2 = 1; // 1 to carrent segment
          out_port = To_port;
          To_port.F1 = 1;
          out_port = To_port;         // shift out REG
        }
      out_port = 0b00100110;    //
      out_port = 0b00110110;    // write to REG_OUT (Ind. is off --> by colomn)

                                // Set to port bits of carrent segment
      To_ind = Out_symbol[Col_];
      To_port = ((To_ind<<1)& 0b00000110) | ((To_ind<<2)& 0xF0); // --> 00xx0xx0
      out_port = To_port ^ 0b00110110;   // column = ~ To_port (Ind. --> on)

      clear(INTCON,TMR0IF);      // clear flag TMR0IF
     }
 }


 //******************** END OF INTERRUPS ***************************************